@@ -35,31 +35,31 @@ csum_partial(const u_char * buff, int len, u_int sum)
 "	jae 1f							\n"			
 "	addl $2, %%ecx						\n"		
 "	jmp 4f							\n"
-"1:	movw (%%esi), %%bx					\n"
+"1:	movw (%%esi), %%di					\n"
 "	addl $2, %%esi						\n"
-"	addw %%bx, %%ax						\n"
+"	addw %%di, %%ax						\n"
 "	adcl $0, %%eax						\n"
 "2:								\n"
 "	movl %%ecx, %%edx					\n"
 "	shrl $5, %%ecx						\n"
 "	jz 2f							\n"
 "	testl %%esi, %%esi					\n"
-"1:	movl (%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 4(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 8(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 12(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 16(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 20(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 24(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
-"	movl 28(%%esi), %%ebx					\n"
-"	adcl %%ebx, %%eax					\n"
+"1:	movl (%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 4(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 8(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 12(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 16(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 20(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 24(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
+"	movl 28(%%esi), %%edi					\n"
+"	adcl %%edi, %%eax					\n"
 "	lea 32(%%esi), %%esi					\n"
 "	dec %%ecx						\n"
 "	jne 1b							\n"
@@ -85,9 +85,9 @@ csum_partial(const u_char * buff, int len, u_int sum)
 "6:	addl %%ecx,%%eax					\n"
 "	adcl $0, %%eax						\n"
 "7: 								\n"
-       : "=a"(sum)
-       : "0"(sum), "c"(len), "S"(buff)
-       : "bx", "dx");
+       : "=a"(sum), "=c"(len), "=S"(buff)
+       : "0"(sum), "1"(len), "2"(buff)
+       : "di", "dx" , "cc");
 
   return (sum);
 }
@@ -127,7 +127,8 @@ inline u_short ip_fast_csum(u_char * iph, u_int ihl)
 	  will assume they contain their original values.
 	*/
 	: "=r" (sum), "=r" (iph), "=r" (ihl)
-	: "1" (iph), "2" (ihl));
+	: "1" (iph), "2" (ihl)
+	: "cc");
   
   return (sum);
 }
@@ -141,7 +142,7 @@ csum_fold(u_int sum)
 "	adcl $0xffff, %0	\n"
 	: "=r" (sum)
 	: "r" (sum << 16), "0" (sum & 0xffff0000)
-	  );
+	: "cc" );
   return ((~sum) >> 16);
 }
  
@@ -159,7 +160,8 @@ csum_tcpudp_magic(u_int saddr, u_int daddr, u_short len,
 "	adcl %3, %0	\n"
 "	adcl $0, %0	\n"
 	: "=r" (sum)
-	: "g" (daddr), "g"(saddr), "g"((ntohs(len) << 16) + proto * 256), "0"(sum));
+	: "g" (daddr), "g"(saddr), "g"((ntohs(len) << 16) + proto * 256), "0"(sum)
+	: "cc");
   return (csum_fold(sum));
 }
 
