@@ -22,15 +22,15 @@ getrnd ()
     }
 
   gettimeofday (&s, 0);
-  srandom (s.tv_usec);
+  srand (s.tv_usec);
   ptr = (u_int *) xor;
-  *ptr = random ();
-  *(ptr + 1) = random ();
-  *(ptr + 2) = random ();
+  *ptr = rand ();
+  *(ptr + 1) = rand ();
+  *(ptr + 2) = rand ();
   ptr = (u_int *) perm;
-  *ptr = random ();
-  *(ptr + 1) = random ();
-  *(ptr + 2) = random ();
+  *ptr = rand ();
+  *(ptr + 1) = rand ();
+  *(ptr + 2) = rand ();
 
 
 }
@@ -61,7 +61,7 @@ mkhash (u_int src, u_short sport, u_int dest, u_short dport)
   *(u_int *) (data + 4) = dest;
   *(u_short *) (data + 8) = sport;
   *(u_short *) (data + 10) = dport;
-  for (i = 0; i < 11; i++)
+  for (i = 0; i < 12; i++)
     res = ( (res << 8) + (data[perm[i]] ^ xor[i])) % 0xff100f;
   return res;
 }
