@@ -134,6 +134,7 @@ atomic_add(int ile, int *co)
 static void
 kfree_skb(struct sk_buff * skb, int type)
 {
+  (void)type;
   free(skb);
 }
 
@@ -193,7 +194,7 @@ static void *
 frag_kmalloc(int size, int dummy)
 {
   void *vp = (void *) malloc(size);
-
+  (void)dummy;
   if (!vp)
     return NULL;
   atomic_add(size, &this_host->ip_frag_mem);
