@@ -43,11 +43,12 @@ static struct proc_node *ip_procs;
 static struct proc_node *udp_procs;
 
 struct proc_node *tcp_procs;
-static int linkoffset = 0;
+static unsigned int linkoffset = 0;
 static int linktype;
 static pcap_t *desc = 0;
 
 char nids_errbuf[PCAP_ERRBUF_SIZE];
+struct pcap_pkthdr * nids_last_pcap_header = NULL;
 
 char *nids_warnings[] = {
     "Murphy - you never should see this message !",
@@ -78,11 +79,14 @@ struct nids_prm nids_params = {
     nids_ip_filter,		/* ip_filter() */
     NULL,			/* pcap_filter */
     1,				/* promisc */
-    0				/* one_loop_less */
+    0,				/* one_loop_less */
+    1024			/* pcap_timeout */
 };
 
 static int nids_ip_filter(struct ip *x, int len)
 {
+    (void)x;
+    (void)len;
     return 1;
 }
 
@@ -180,7 +184,10 @@ static void pcap_hand(u_char * par, struct pcap_pkthdr *hdr, u_char * data)
     u_char *data_aligned;
 #ifdef DLT_IEEE802_11
     unsigned short fc;
+    int linkoffset_tweaked_by_prism_code = 0;
 #endif
+    nids_last_pcap_header = hdr;
+    (void)par; /* warnings... */
     switch (linktype) {
     case DLT_EN10MB:
 	if (hdr->caplen < 14)
@@ -196,12 +203,23 @@ static void pcap_hand(u_char * par, struct pcap_pkthdr *hdr, u_char * data)
 	    /* non-ip frame */
 	    return;
 	break;
+#ifdef DLT_PRISM_HEADER
+#ifndef DLT_IEEE802_11
+#error DLT_PRISM_HEADER is defined, but DLT_IEEE802_11 is not ???
+#endif
+    case DLT_PRISM_HEADER:
+	linkoffset = 144; //sizeof(prism2_hdr);
+	linkoffset_tweaked_by_prism_code = 1;
+        //now let DLT_IEEE802_11 do the rest
+#endif
 #ifdef DLT_IEEE802_11
     case DLT_IEEE802_11:
 	/* I don't know why frame control is always little endian, but it 
 	 * works for tcpdump, so who am I to complain? (wam)
 	 */
-	fc = EXTRACT_LE_16BITS(data);
+	if (!linkoffset_tweaked_by_prism_code)
+		linkoffset = 0;
+	fc = EXTRACT_LE_16BITS(data + linkoffset);
 	if (FC_TYPE(fc) != T_DATA || FC_WEP(fc)) {
 	    return;
 	}
@@ -209,9 +227,9 @@ static void pcap_hand(u_char * par, struct pcap_pkthdr *hdr, u_char * data)
 	    /* a wireless distribution system packet will have another
 	     * MAC addr in the frame
 	     */
-	    linkoffset = 30;
+	    linkoffset += 30;
 	} else {
-	    linkoffset = 24;
+	    linkoffset += 24;
 	}
 	if (hdr->len < linkoffset + LLC_FRAME_SIZE)
 	    return;
@@ -258,7 +276,7 @@ static void gen_ip_frag_proc(u_char * data, int len)
     if (!nids_params.ip_filter(iph, len))
 	return;
 
-    if (len < sizeof(struct ip) || iph->ip_hl < 5 || iph->ip_v != 4 ||
+    if (len < (int)sizeof(struct ip) || iph->ip_hl < 5 || iph->ip_v != 4 ||
 	ip_fast_csum((unsigned char *) iph, iph->ip_hl) != 0 ||
 	len < ntohs(iph->ip_len) || ntohs(iph->ip_len) < iph->ip_hl << 2) {
 	nids_params.syslog(NIDS_WARN_IP, NIDS_WARN_IP_HDR, iph, 0);
@@ -311,11 +329,11 @@ static void process_udp(char *data)
     int hlen = iph->ip_hl << 2;
     int len = ntohs(iph->ip_len);
     int ulen;
-    if (len - hlen < sizeof(struct udphdr))
+    if (len - hlen < (int)sizeof(struct udphdr))
 	return;
     udph = (struct udphdr *) (data + hlen);
     ulen = ntohs(udph->UH_ULEN);
-    if (len - hlen < ulen || ulen < sizeof(struct udphdr))
+    if (len - hlen < ulen || ulen < (int)sizeof(struct udphdr))
 	return;
     if (my_udp_check
 	((void *) udph, ulen, iph->ip_src.s_addr,
@@ -402,7 +420,7 @@ static int open_live()
 	promisc = (nids_params.promisc != 0);
 
     if ((desc = pcap_open_live(device, 16384, promisc,
-			       1024, nids_errbuf)) == NULL)
+			       nids_params.pcap_timeout, nids_errbuf)) == NULL)
 	return 0;
 #ifdef __linux__
     if (!strcmp(device, "any") && nids_params.promisc
@@ -440,6 +458,9 @@ int nids_init()
     }
     switch ((linktype = pcap_datalink(desc))) {
 #ifdef DLT_IEEE802_11
+#ifdef DLT_PRISM_HEADER
+    case DLT_PRISM_HEADER:
+#endif
     case DLT_IEEE802_11:
 	/* wireless, need to calculate offset per frame */
 	break;
@@ -472,6 +493,11 @@ int nids_init()
     case DLT_FDDI:
         linkoffset = 21;
         break;
+#endif        
+#ifdef DLT_PPP_SERIAL 
+    case DLT_PPP_SERIAL:
+        linkoffset = 4;
+        break;
 #endif        
     default:
 	strcpy(nids_errbuf, "link type unknown");
