@@ -5,17 +5,19 @@
 
 #ifndef _NIDS_NIDS_H
 #define _NIDS_NIDS_H
-
+#define NIDS_MAJOR 1
+#define NIDS_MINOR 19
 #include <sys/types.h>
-
-enum {
+enum
+{
   NIDS_WARN_IP = 1,
   NIDS_WARN_TCP,
   NIDS_WARN_UDP,
   NIDS_WARN_SCAN
 };
 
-enum {
+enum
+{
   NIDS_WARN_UNDEFINED = 0,
   NIDS_WARN_IP_OVERSIZED,
   NIDS_WARN_IP_INVLIST,
@@ -33,15 +35,18 @@ enum {
 #define NIDS_CLOSE 3
 #define NIDS_RESET 4
 #define NIDS_TIMED_OUT 5
+#define NIDS_EXITING   6	/* nids is exiting; last chance to get data */
 
-struct tuple4 {
+struct tuple4
+{
   u_short source;
   u_short dest;
-  u_long saddr;
-  u_long daddr;
+  u_int saddr;
+  u_int daddr;
 };
 
-struct half_stream {
+struct half_stream
+{
   char state;
   char collect;
   char collect_urg;
@@ -63,12 +68,14 @@ struct half_stream {
   u_char urg_seen;
   u_int urg_ptr;
   u_short window;
-
+  u_char ts_on;
+  u_int curr_ts; 
   struct skbuff *list;
   struct skbuff *listtail;
 };
 
-struct tcp_stream {
+struct tcp_stream
+{
   struct tuple4 addr;
   char nids_state;
   struct lurker_node *listeners;
@@ -83,34 +90,40 @@ struct tcp_stream {
   struct tcp_stream *next_free;
 };
 
-struct nids_prm {
+struct nids_prm
+{
   int n_tcp_streams;
   int n_hosts;
   char *device;
+  char *filename;
   int sk_buff_size;
   int dev_addon;
-  void (*syslog)();
+  void (*syslog) ();
   int syslog_level;
   int scan_num_hosts;
   int scan_delay;
   int scan_num_ports;
-  void (*no_mem)(char *);
-  int (*ip_filter)();
+  void (*no_mem) (char *);
+  int (*ip_filter) ();
   char *pcap_filter;
+  int promisc;
+  int one_loop_less;
+  int pcap_timeout;
 };
 
-int nids_init();
-void nids_register_ip_frag(void (*));
-void nids_register_ip(void (*));
-void nids_register_tcp(void (*));
-void nids_killtcp(struct tcp_stream *);
-void nids_discard(struct tcp_stream *, int);
-void nids_run();
-int nids_getfd();
-int nids_next();
+int nids_init ();
+void nids_register_ip_frag (void (*));
+void nids_register_ip (void (*));
+void nids_register_tcp (void (*));
+void nids_register_udp (void (*));
+void nids_killtcp (struct tcp_stream *);
+void nids_discard (struct tcp_stream *, int);
+void nids_run ();
+int nids_getfd ();
+int nids_next ();
 
 extern struct nids_prm nids_params;
 extern char *nids_warnings[];
 extern char nids_errbuf[];
-
+extern struct pcap_pkthdr *last_pcap_header;
 #endif /* _NIDS_NIDS_H */
