@@ -125,7 +125,7 @@ static int get_ts(struct tcphdr * this_tcphdr, unsigned int * ts)
   unsigned int tmp_ts;
   unsigned char * options = (char*)(this_tcphdr + 1);
   int ind = 0, ret = 0;
-  while (ind <=  len - sizeof (struct tcphdr) - 10 )
+  while (ind <=  len - (int)sizeof (struct tcphdr) - 10 )
   	switch (options[ind]) {
 		case 0: /* TCPOPT_EOL */
 			return ret;
@@ -408,7 +408,7 @@ tcp_queue(struct tcp_stream * a_tcp, struct tcphdr * this_tcphdr,
       while (pakiet) {
 	if (after(pakiet->seq, EXP_SEQ))
 	  break;
-	if (after(pakiet->seq + pakiet->len, EXP_SEQ)) {
+	if (after(pakiet->seq + pakiet->len + pakiet->fin, EXP_SEQ)) {
 	  struct skbuff *tmp;
 	  
 	  add_from_skb(a_tcp, rcv, snd, pakiet->data,
@@ -493,8 +493,7 @@ prune_queue(struct half_stream * rcv, struct tcphdr * this_tcphdr)
 }
 
 static void
-handle_ack(struct tcp_stream * a_tcp, struct half_stream * snd,
-	   struct half_stream * rcv, u_int acknum)
+handle_ack(struct half_stream * snd, u_int acknum)
 {
   int ackdiff;
 
@@ -580,7 +579,7 @@ process_tcp(u_char * data, int skblen)
 
   ugly_iphdr = this_iphdr;
   iplen = ntohs(this_iphdr->ip_len);
-  if (iplen - 4 * this_iphdr->ip_hl <sizeof(struct tcphdr)) {
+  if ((unsigned)iplen < 4 * this_iphdr->ip_hl + sizeof(struct tcphdr)) {
     nids_params.syslog(NIDS_WARN_TCP, NIDS_WARN_TCP_HDR, this_iphdr,
 		       this_tcphdr);
     return;
@@ -643,9 +642,14 @@ process_tcp(u_char * data, int skblen)
     } else a_tcp->server.ts_on = 0;	
     return;
   }
-  if (!before(ntohl(this_tcphdr->th_seq), rcv->ack_seq + rcv->window) ||
-      before(ntohl(this_tcphdr->th_seq) + datalen, rcv->ack_seq))
-    return;
+  if (
+  	! (  !datalen && ntohl(this_tcphdr->th_seq) == rcv->ack_seq  )
+  	&&
+  	( !before(ntohl(this_tcphdr->th_seq), rcv->ack_seq + rcv->window) ||
+          before(ntohl(this_tcphdr->th_seq) + datalen, rcv->ack_seq)  
+        )
+     )     
+     return;
 
   if ((this_tcphdr->th_flags & TH_RST)) {
     if (a_tcp->nids_state == NIDS_DATA) {
@@ -723,7 +727,7 @@ process_tcp(u_char * data, int skblen)
     }
   }
   if ((this_tcphdr->th_flags & TH_ACK)) {
-    handle_ack(a_tcp, snd, rcv, ntohl(this_tcphdr->th_ack));
+    handle_ack(snd, ntohl(this_tcphdr->th_ack));
     if (rcv->state == FIN_SENT)
       rcv->state = FIN_CONFIRMED;
     if (rcv->state == FIN_CONFIRMED && snd->state == FIN_CONFIRMED) {
@@ -769,8 +773,8 @@ int
 tcp_init(int size)
 {
   int i;
-  if (!nids_params.n_tcp_streams) return 0;
-  tcp_stream_table_size = nids_params.n_tcp_streams;
+  if (!size) return 0;
+  tcp_stream_table_size = size;
   tcp_stream_table = malloc(tcp_stream_table_size * sizeof(char *));
   if (!tcp_stream_table)
     nids_params.no_mem("tcp_init");
@@ -818,7 +822,10 @@ process_icmp(u_char * data)
   struct lurker_node *i;
 
   int from_client;
-  int len = ntohs(iph->ip_len) - (iph->ip_hl << 2);
+  /* we will use unsigned, to suppress warning; we must be careful with
+     possible wrap when substracting 
+     the following is ok, as the ip header has already been sanitized */
+  unsigned int len = ntohs(iph->ip_len) - (iph->ip_hl << 2);
   
   if (len < sizeof(STRUCT_ICMP))
     return;
@@ -827,16 +834,18 @@ process_icmp(u_char * data)
     return;
   if (pkt->ICMP_TYPE != ICMP_DEST_UNREACH)
     return;
-  len -= 8;
+  /* ok due to check 7 lines above */  
+  len -= sizeof(STRUCT_ICMP);
   // sizeof(struct icmp) is not what we want here
   
   if (len < sizeof(struct ip))
     return;
 
   orig_ip = (struct ip *) (((char *) pkt) + 8);
+  if (len < (unsigned)(orig_ip->ip_hl << 2) + 8)
+     return;
+  /* subtraction ok due to the check above */
   len -= orig_ip->ip_hl << 2;
-  if (len < 8)
-    return;
   if ((pkt->ICMP_CODE & 15) == ICMP_PROT_UNREACH ||
       (pkt->ICMP_CODE & 15) == ICMP_PORT_UNREACH)
     match_addr = 1;
